from .wxControl import *
import wx.lib.agw.gradientbutton as gradientbutton

class 图文按钮L(gradientbutton.GradientButton, 公用方法):
    pass

    def 是否允许通过单击获取焦点(self):
        return self.AcceptsFocus()

    def 取最佳尺寸(self):
        '返回一个元组,(宽，高),覆盖基类虚拟。根据标签和边框尺寸确定按钮的最佳尺寸。'
        return self.DoGetBestSize()

    def 取渐变底纹底端颜色(self):
        return self.GetBottomEndColour()

    def 取渐变底纹顶端颜色(self):
        return self.GetTopEndColour()

    def 取渐变底纹底部起始颜色(self):
        return self.GetBottomStartColour()

    def 取渐变底纹按下底部起始颜色(self):
        return self.GetPressedBottomColour()

    def 取渐变底纹按下顶部起始颜色(self):
        return self.GetPressedTopColour()

    def 取渐变阴影顶部起始颜色(self):
        return self.GetTopStartColour()

    @组件_异常检测
    def 置各状态颜色(self,起始色,前景色):
        '设置底部，顶部，按下和前景色,起始色–用于底部，顶部和压制的基础颜色,前景色 –用于文本的颜色'
        return self.SetBaseColours(起始色,前景色)

    @组件_异常检测
    def 置图片(self,图片):
        return self.SetBitmapLabel(图片)

    @组件_异常检测
    def 置渐变底纹底端颜色(self,颜色):
        return self.SetBottomEndColour(颜色)

    @组件_异常检测
    def 置渐变底纹顶部底部颜色(self,颜色):
        return self.SetBottomStartColour(颜色)

    @组件_异常检测
    def 置默认按钮(self):
        return self.SetDefault()

    @组件_异常检测
    def 置最佳尺寸(self):
        '将按钮调整为最佳尺寸'
        self.SetInitialSize()

    @组件_异常检测
    def 置渐变阴影下底部开始颜色(self,颜色):
        return self.SetPressedBottomColour(颜色)

    @组件_异常检测
    def 置渐变阴影设置按下顶部起始颜色(self,颜色):
        return self.SetPressedTopColour(颜色)

    @组件_异常检测
    def 置渐变底纹顶端颜色(self,颜色):
        return self.SetTopEndColour(颜色)

    @组件_异常检测
    def 置渐变底纹顶部起始颜色(self,颜色):
        return self.SetTopStartColour(颜色)



import unittest

from .stringUtil import *


class TestStringUtil(unittest.TestCase):

    def test_1(self):
        data = strCut("123abc123", "123$123")
        self.assertEqual(data, "abc")
        data = strCut("123abc456", "abc$")
        self.assertEqual(data, "456")
        data = strCut("123abc456", "$abc")
        self.assertEqual(data, "123")

        data = 文本_取左边("123abc456", "abc")
        self.assertEqual(data, "123")

        data = 文本_取右边("123abc456", "abc")
        self.assertEqual(data, "456")

        data = 文本_取出中间文本("123abc456", "123", "456")
        self.assertEqual(data, "abc")


    def test_2(self):
        data = 文本_取随机字母(10, 0)
        print(data)
        self.assertNotEqual(data, "")

        data = 文本_取随机字母(10, 1)
        print(data)
        self.assertNotEqual(data, "")

        data = 文本_取随机字母(10, 2)
        print(data)
        self.assertNotEqual(data, "")

        data = 文本_取随机字母和数字(10)
        print(data)
        self.assertNotEqual(data, "")

        data = 文本_取随机数字(10)
        print(data)
        self.assertNotEqual(data, "")

        data = 文本_取随机汉字(10)
        print(data)
        self.assertNotEqual(data, "")

        data = 文本_取随机姓氏()
        print(data)
        self.assertNotEqual(data, "")

        data = 文本_取随机姓氏(True)
        print(data)
        self.assertNotEqual(data, "")

        data = 文本_取随机手机号()
        print(data)
        self.assertNotEqual(data, "")

        data = 文本_取随机邮箱()
        print(data)
        self.assertNotEqual(data, "")

    def test_3(self):
        data = 文本_删左边("123abc123", 3)
        self.assertEqual(data, "abc123")
        data = 文本_删右边("123abc123", 3)
        self.assertEqual(data, "123abc")
        data = 文本_删中间("123abc123", 3, 3)
        self.assertEqual(data, "123123")

        data = 文本区分_只取字母("123abc123", 0)
        self.assertEqual(data, "abc")

        data = 文本区分_只取数字("123abc123")
        self.assertEqual(data, "123123")

        data = 判断文本("123abc123", ["123"])
        self.assertEqual(data, True)

        data = 判断文本s("123abc123", ["123", "abc"])
        self.assertEqual(data, "123")

        data = 判断文本("你好", ["123"])
        self.assertEqual(data, False)

        data = 判断文本s("123abc123你好", ["aaa", "你好"])
        self.assertEqual(data, "你好")

        data = 文本_取手机号码("手机号:13588899988 手机号:13588899987")
        self.assertEqual(data, ['13588899988', '13588899987'])

        data = 文本_取IP地址("192.168.1.1")
        self.assertEqual(data, ['192.168.1.1'])

        data = 文本_取电话号码("010-33366669")
        self.assertEqual(data, ['010-33366669'])

        data = 文本_取QQ号码("666655555")
        self.assertEqual(data, ['666655555'])

        data = 文本_取邮政编码("520420")
        self.assertEqual(data, ['520420'])

        data = 文本_取身份证号码("44666655559999666")
        self.assertEqual(data, ['44666655559999666'])

        data = 文本_取双字节字符("你好hello123,./")
        self.assertEqual(data, "你好")

        data = 文本_取网址("http://www.github,com")
        self.assertEqual(data, ['http://www.github,com'])

        data = 文本_取IP跟端口("192.168.0.1:9998")
        self.assertEqual(data, ['192.168.0.1:9998'])

        data = 文本_取邮箱号码("666666@qq.com")
        self.assertEqual(data, ['666666@qq.com'])

        data = 文本_大小写翻转("abc ABC")
        self.assertEqual(data, 'ABC abc')

        data = 文本_是否为大写字母("abc ABC")
        self.assertEqual(data, False)

        data = 文本_是否为大写字母("ABC")
        self.assertEqual(data, True)

        data = 文本_是否为小写字母("abc")
        self.assertEqual(data, True)

        data = 文本_是否为字母("abcABC")
        self.assertEqual(data, True)

        data = 文本_是否为字母("abcABC123")
        self.assertEqual(data, False)

        data = 文本_是否为数字字母("abcABC123")
        self.assertEqual(data, True)

        data = 文本_是否为数字("abc123465")
        self.assertEqual(data, False)

        data = 文本_是否为数字字母("123465")
        self.assertEqual(data, True)

        data = 文本_取出现次数("AAAaaaBBBbbb", "b")
        self.assertEqual(data, 3)

        data = 文本_单词首字母大写("apple")
        self.assertEqual(data, "Apple")

        data = 文本_填充空格_居中("apple", 10)
        self.assertEqual(data, "  apple   ")

        data = 文本_填充空格_左对齐("apple", 10)
        self.assertEqual(data, "apple     ")

        data = 文本_填充空格_右对齐("apple", 10)
        self.assertEqual(data, "     apple")

        data = 文本_自动补零("apple", 10)
        self.assertEqual(data, "00000apple")

        data = 数组_转文本(["a", "b", "c"], "-")
        self.assertEqual(data, "a-b-c")

        data = 文本_判断文本前缀("pro_apple", "pro")
        self.assertEqual(data, True)

        data = 文本_判断文本后缀("pro_apple", "apple")
        self.assertEqual(data, True)

        data = 文本_TAB转空格("\tpro_apple", 4)
        self.assertEqual(data, "    pro_apple")

        data = 文本_取随机IP()
        self.assertNotEqual(data, "")
        print(data)

    def test_4(self):
        data = 文本_取出文本中汉字("你好,祖国,hello")
        self.assertEqual(data, "你好祖国")

        data = 文本_逐字分割("你好,祖国,hello")
        self.assertEqual(data, ['你', '好', ',', '祖', '国', ',', 'h', 'e', 'l', 'l', 'o'])

        data = 文本_颠倒("你好,祖国,hello")
        self.assertEqual(data, "olleh,国祖,好你")

        data = 文本_是否为汉字("你好祖国")
        self.assertEqual(data, True)

        data = 文本_是否为汉字("你好祖国123")
        self.assertEqual(data, False)

        data = 文本_取中间_批量("abc123efg,abc123efg,abc123efg", "abc", "efg")
        self.assertEqual(data, ['123', '123', '123'])

    def test_5(self):
        data = 文本_汉字转拼音("祖国你好", " ")
        print(data)



"""

.. Hint::
    时钟

    易语言的时钟组件

    实现了 时钟周期事件

    使用装饰器设置定时任务 函数内返回 false则停止时钟

.. literalinclude:: ../../../pyefun/clockUtil_test.py
    :language: python
    :caption: 代码示例
    :linenos:
    :lines: 1-100

.. code-block:: python
   :linenos:

    @时钟周期事件(时钟周期=1000)
    def 定时任务():
        print("定时任务")
        # return True
        return False

    定时任务2运行状态 = True

    def 定时任务2():
        print(1111111111111,定时任务2运行状态)
        return 定时任务2运行状态
        # return False


    时钟(定时任务2, 时钟周期=2000)

    @时钟周期事件(时钟周期=5000)
    def 定时任务3():
        global 定时任务2运行状态
        print("把时钟停止掉")
        定时任务2运行状态 = False
        return False


"""


from threading import Timer



def 时钟(func, 时钟周期):
    """
    时钟 定时调用函数  也可以使用装饰器 @时钟周期事件(时钟周期=1000)

    def 定时任务():
        print("定时任务")
        return False 停止时钟周期事件

    时钟(定时任务, 时钟周期=2000)

    :param func: 运行函数 函数返回false 则停止时钟周期事件
    :param 时钟周期: 毫秒
    """
    时钟周期 = int(时钟周期 / 1000)
    def 自调用函数():
        if not func() == False:
            return Timer(时钟周期, 自调用函数).start()
    return Timer(时钟周期, 自调用函数).start()

def 时钟周期事件(*args, **kwargs):
    """
    装饰器

    @时钟周期事件(时钟周期=1000)
    def 定时任务():
        print("定时任务")
    """
    def warpp(func):
        时钟(func, kwargs['时钟周期'])
    return warpp





import unittest
import warnings

from .seleniumUtil import *
from pyefun import *


class TestseleniumUtil(unittest.TestCase):

    def test_1(self):
        pass
        浏览器初始化本地环境()

        浏览器 = 浏览器类()
        浏览器.打开chrome()

        浏览器.浏览网页("https://www.baidu.com")
        print(浏览器.取页面标题())
        print(浏览器.取名称())
        # print(浏览器.取页面链接())
        # print(浏览器.取页面源码())
        with 时间统计("就绪"):
            print(浏览器.等待元素显示(5, "s_ipt"))

        延时(1)

        搜索输入框 = 浏览器.取元素从id("kw")
        百度一下按钮 = 浏览器.取元素从id("su")

        搜索输入框.输入("pyefun")
        百度一下按钮.点击()

        延时(1)

        with 时间统计("结果就绪"):
            print(浏览器.等待元素显示(5, "result"))

        搜索结果 = 浏览器.取元素从class_name("result")
        print("搜索结果多少条", len(搜索结果))
        for 元素 in 搜索结果:
            print(元素.取文本())
            # print(元素.取源码())

        延时(1)
        浏览器.退出()

    def test_2(self):
        pass

        warnings.simplefilter('ignore', ResourceWarning) # 屏蔽一堆错误

        浏览器 = 浏览器类()
        远程浏览器地址 = "http://127.0.0.1:4444/wd/hub"
        while 浏览器.远程浏览器是否就绪(远程浏览器地址) == False:
            延时(1)
            print("浏览器未就绪")

        浏览器.获取远程chrome(远程浏览器地址)

        浏览器.浏览网页("https://www.baidu.com")
        print(浏览器.取页面标题())
        print(浏览器.取名称())
        # print(浏览器.取页面链接())
        # print(浏览器.取页面源码())
        with 时间统计("就绪"):
            print(浏览器.等待元素显示(5, "s_ipt"))

        延时(1)

        搜索输入框 = 浏览器.取元素从id("kw")
        百度一下按钮 = 浏览器.取元素从id("su")

        搜索输入框.输入("pyefun")
        百度一下按钮.点击()

        延时(1)

        with 时间统计("结果就绪"):
            print(浏览器.等待元素显示(5, "result"))

        搜索结果 = 浏览器.取元素从class_name("result")
        print("搜索结果多少条", len(搜索结果))
        for 元素 in 搜索结果:
            print(元素.取文本())
            # print(元素.取源码())

        延时(1)
        浏览器.退出()



import os
import sys
import zipfile

#
# def rename(pwd: str, filename=''):
#     """压缩包内部文件有中文名, 解压后出现乱码，进行恢复"""
#
#     path = f'{pwd}/{filename}'
#     if os.path.isdir(path):
#         for i in os.scandir(path):
#             rename(path, i.name)
#     newname = filename.encode('cp437').decode('gbk')
#     os.rename(path, f'{pwd}/{newname}')


"""
zip文件解压缩

https://docs.python.org/zh-cn/3.9/library/zipfile.html?highlight=zipfile#


"""

import zipfile
from pyefun import *
import pyefun.commonlyUtil as commonlyUtil

@异常处理返回类型逻辑型
def zip解压(压缩包的路径: str, 解压路径: str):
    file = zipfile.ZipFile(压缩包的路径)
    if 文件是否存在(解压路径) == False:
        commonlyUtil.目录_创建(解压路径)
    file.extractall(解压路径)
    file.close()


@异常处理返回类型逻辑型
def zip压缩(保存压缩包的路径: str, 欲压缩文件或者文件夹: str,mode="w"):
    pass
    保存压缩包的路径 = 路径_优化路径(保存压缩包的路径)
    欲压缩文件或者文件夹 = 路径_优化路径(欲压缩文件或者文件夹)
    路径字符长度 = 取文本长度(欲压缩文件或者文件夹)
    if 文件_是否为文件(欲压缩文件或者文件夹):
        with zipfile.ZipFile(保存压缩包的路径, mode=mode) as target:
            target.write(欲压缩文件或者文件夹, 文件_取文件名(欲压缩文件或者文件夹))
        return True
    if 文件_是否为目录(欲压缩文件或者文件夹):
        with zipfile.ZipFile(保存压缩包的路径, mode=mode) as target:
            压缩文件列表 = []
            for 路径信息 in os.walk(欲压缩文件或者文件夹):
                for 文件名 in 路径信息[2]:
                    目录 = 路径信息[0]
                    文件路径 = 路径_合并(目录, 文件名)
                    相对路径 = 取文本右边(文件路径, 取文本长度(文件路径) - 路径字符长度)
                    if(文件路径 == 保存压缩包的路径):
                        continue
                        # 防止死循环一直压缩
                    压缩文件列表.append([文件路径,相对路径])

            # 开始压缩
            for item in 压缩文件列表:
                文件路径 = item[0]
                相对路径 = item[1]
                target.write(文件路径, 相对路径)
        return True



import wx.adv
import wx
from .wxControl import *


class 时钟(wx.Timer, 公用方法):
    pass

    def 启动(self, milliseconds=-1, oneShot=False):
        self.Start(milliseconds, oneShot)

    def 停止(self):
        self.Stop()



import wx.adv
import wx
from .wxControl import *

class 时间框(wx.adv.TimePickerCtrl, 公用方法):
    pass

    def 取当前时间(self):
        '返回一个元组，(时,分,秒)'
        return self.GetTime()

    @组件_异常检测
    def 置当前时间(self,时,分,秒):
        return self.SetTime(时,分,秒)

    def 取当前时间_dt(self):
        '返回wx.DateTime'
        return self.GetValue()

    @组件_异常检测
    def 置当前时间_dt(self,时间):
        'wx.DateTime格式时间'
        return self.SetValue(时间)




# -*- coding:utf-8 -*-
from win32api import GetSystemMetrics
import win32gui,win32con,win32api,os,re,subprocess
import win32clipboard as w
from .public import *

#字母键
按键_A,按键_B,按键_C,按键_D,按键_E,按键_F,按键_G,按键_H,按键_I,按键_J,按键_K,按键_L,按键_M,按键_N,按键_O,按键_P,按键_Q,按键_R,按键_S,按键_T,按键_U,按键_V,按键_W,按键_X,按键_Y,按键_Z = 65,66,67,68,69,70,71,72,73,74,75,76,77,78,79,80,81,82,83,84,85,86,87,88,89,90
#字母上的数字键(左边)
按键_0_,按键_1_,按键_2_,按键_3_,按键_4_,按键_5_,按键_6_,按键_7_,按键_8_,按键_9_ = 48,49,50,51,52,53,54,55,56,57
#数字键(右边)
按键_0,按键_1,按键_2,按键_3,按键_4,按键_5,按键_6,按键_7,按键_8,按键_9 = 96,97,98,99,100,101,102,103,104,105
#功能键
按键_F1,按键_F2,按键_F3,按键_F4,按键_F5,按键_F6,按键_F7,按键_F8,按键_F9,按键_F10,按键_F11,按键_F12=112,113,114,115,116,117,118,119,120,121,122,123
#其它键
按键_乘,按键_加,按键_小回车,按键_减,按键_点,按键_除 = 106,107,108,109,110,111
按键_退格,按键_TAB,按键_清除键,按键_回车,按键_SHIFT,按键_CONTROL,按键_ALT,按键_大小写键,按键_ESC,按键_空格,按键_PAGE_UP,按键_PAGE_DOWN,按键_END,按键_HOME,按键_向左,按键_向右=8,9,12,13,16,17,18,20,27,32,33,34,35,36,37,38
按键_向右,按键_向下,按键_INS,按键_DEL,按键_NUM,按键_冒号键,按键_等号键,按键_逗号键,按键_下划线键,按键_句号键,按键_问号键,按键_波浪键,按键_左括号,按键_斜杠键,按键_右括号,按键_引号键=39,40,45,46,144,186,187,188,189,190,191,192,219,220,221,222
按键_音量加,按键_音量减,按键_停止,按键_静音,按键_浏览器,按键_邮件,按键_搜索,按键_搜餐 = 175,174,179,173,172,180,170,171
按键_WIN = 91

@异常处理返回类型逻辑型
def 窗口_置父窗口(子窗口句柄,父窗口句柄):
    '将子窗口嵌入父窗口内'
    return win32gui.SetParent(子窗口句柄, 父窗口句柄)


@异常处理返回类型逻辑型
def 鼠标_移动(x,y):
    '在桌面上移动鼠标位置'
    win32api.SetCursorPos((x, y))


@异常处理返回类型逻辑型
def 鼠标_点击(按键=0,类型=0,x=0,y=0):
    '''
    模拟鼠标点击
    :param 按键: 0.左键 1.中键 2.右键
    :param 类型: 0.单击 1.按下 2.弹起
    :return:
    '''
    代码 = {(0,1):2,(0,2):4,(1,1):32,(1,2):64,(2,1):8,(2,2):16}
    按键类型 = {1:(32,64),2:(8,16)}
    if not (按键,类型) in 代码:
        键码 = 按键类型[按键] if 按键 in 按键类型 else (2,4)
        win32api.mouse_event(键码[0],0,0,0,0)
        win32api.mouse_event(键码[1],0,0,0,0)
    else:
        win32api.mouse_event(代码[(按键,类型)],0,0,0,0)


@异常处理返回类型逻辑型
def 键盘_点击(键码,类型=0):
    '''
    :param 键码: 可直接使用 按键_按键名
    :param 类型: 0.单击 1.按下 2.弹起
    :return:
    '''
    if 类型 == 1:
        win32api.keybd_event(键码, 0, 0, 0)
    elif 类型 == 2:
        win32api.keybd_event(键码, 0, win32con.KEYEVENTF_KEYUP, 0)
    else:
        win32api.keybd_event(键码, 0, 0, 0)
        win32api.keybd_event(键码, 0, win32con.KEYEVENTF_KEYUP, 0)


@异常处理返回类型逻辑型
def 窗口_取窗口句柄(类名=None,标题=None):
    '返回第一个找到的句柄'
    return win32gui.FindWindow(类名, 标题)


@异常处理返回类型逻辑型
def 窗口_取坐标处窗口句柄(x,y):
    return win32gui.WindowFromPoint((x,y))


@异常处理返回类型逻辑型
def 窗口_取窗口类名(窗口句柄):
    return win32gui.GetClassName(窗口句柄)


@异常处理返回类型逻辑型
def 窗口_取窗口标题(窗口句柄):
    return win32gui.GetWindowText(窗口句柄)


@异常处理返回类型逻辑型
def 窗口_取窗口矩形(窗口句柄):
    '成功返回窗口 左边 顶边 右边 底边'
    return win32gui.GetWindowRect(窗口句柄)


@异常处理返回类型逻辑型
def 窗口_枚举子窗口句柄(窗口句柄):
    '成功返回子窗口句柄列表'
    hwndChildList = []
    win32gui.EnumChildWindows(窗口句柄, lambda hwnd, param: param.append(hwnd), hwndChildList)
    return hwndChildList


@异常处理返回类型逻辑型
def 窗口_发送信息(窗口句柄,消息类型,参数1=0,参数2=0):
    return win32gui.PostMessage(窗口句柄,消息类型,参数1,参数2)


@异常处理返回类型逻辑型
def 窗口_发送信息2(窗口句柄,消息类型,参数1=0,参数2=0):
    return win32gui.SendMessage(窗口句柄,消息类型,参数1,参数2)


@异常处理返回类型逻辑型
def 窗口_发送文本(窗口句柄,内容):
    return win32api.SendMessage(窗口句柄, win32con.WM_SETTEXT, 0, 内容)

@异常处理返回类型逻辑型
def 窗口_取窗口文本长度(窗口句柄):
    return win32api.SendMessage(窗口句柄, win32con.WM_GETTEXTLENGTH, 0, 0) +1


@异常处理返回类型逻辑型
def 窗口_取窗口文本内容(窗口句柄):
    bufSize = win32api.SendMessage(窗口句柄, win32con.WM_GETTEXTLENGTH, 0, 0) + 1
    strBuf = win32gui.PyMakeBuffer(bufSize)
    length = win32gui.SendMessage(窗口句柄, win32con.WM_GETTEXT, bufSize, strBuf)
    address, length = win32gui.PyGetBufferAddressAndLen(strBuf)
    text = win32gui.PyGetString(address, length)
    return text[:-1]


@异常处理返回类型逻辑型
def 窗口_关闭窗口(窗口句柄):
    return win32gui.PostMessage(窗口句柄, win32con.WM_CLOSE, 0, 0)


@异常处理返回类型逻辑型
def 窗口_是否最小化(窗口句柄):
    return bool(win32gui.IsIconic(窗口句柄))


@异常处理返回类型逻辑型
def 窗口_最大化(窗口句柄):
    return bool(win32gui.ShowWindow(窗口句柄,win32con.SW_MAXIMIZE))


@异常处理返回类型逻辑型
def 窗口_最大化2(窗口句柄):
    '激活窗口并将其最大化'
    return bool(win32gui.ShowWindow(窗口句柄,win32con.SW_SHOWMAXIMIZED))


@异常处理返回类型逻辑型
def 窗口_最小化(窗口句柄):
    return bool(win32gui.ShowWindow(窗口句柄, win32con.SW_MINIMIZE))


@异常处理返回类型逻辑型
def 窗口_最小化2(窗口句柄):
    '激活窗口并将其最小化'
    return bool(win32gui.ShowWindow(窗口句柄, win32con.SW_SHOWMINIMIZED))


@异常处理返回类型逻辑型
def 窗口_最小化3(窗口句柄):
    '窗口最小化，激活窗口仍然维持激活状态'
    return bool(win32gui.ShowWindow(窗口句柄, win32con.SW_SHOWMINNOACTIVE))


@异常处理返回类型逻辑型
def 窗口_隐藏窗口(窗口句柄):
    return bool(win32gui.ShowWindow(窗口句柄, win32con.SW_HIDE))


@异常处理返回类型逻辑型
def 窗口_显示窗口(窗口句柄):
    return bool(win32gui.ShowWindow(窗口句柄, win32con.SW_SHOW))


@异常处理返回类型逻辑型
def 窗口_显示窗口2(窗口句柄):
    '以原尺寸恢复显示'
    return bool(win32gui.ShowWindow(窗口句柄, win32con.SW_RESTORE))


@异常处理返回类型逻辑型
def 窗口_显示窗口3(窗口句柄):
    '以窗口原来的状态显示窗口。激活窗口仍然维持激活状态'
    return bool(win32gui.ShowWindow(窗口句柄, win32con.SW_SHOWNA))


@异常处理返回类型逻辑型
def 窗口_显示窗口4(窗口句柄):
    '以窗口最近一次的大小和状态显示窗口。激活窗口仍然维持激活状态'
    return bool(win32gui.ShowWindow(窗口句柄, win32con.SW_SHOWNOACTIVATE))


@异常处理返回类型逻辑型
def 窗口_显示窗口5(窗口句柄):
    '激活并显示一个窗口。如果窗口被最小化或最大化，系统将其恢复到原来的尺寸和大小。应用程序在第一次显示窗口的时候应该指定此标志'
    return bool(win32gui.ShowWindow(窗口句柄, win32con.SW_SHOWNORMAL))


@异常处理返回类型逻辑型
def 窗口_调整(窗口句柄,左边,顶边,宽度,高度):
    '移动窗口 调整宽度高度'
    return win32gui.MoveWindow(窗口句柄,左边,顶边,宽度,高度,True)


@异常处理返回类型逻辑型
def 窗口_调整2(窗口句柄,左边,顶边,宽度,高度):
    '移动窗口 调整宽度高度'
    return win32gui.SetWindowPos(窗口句柄, win32con.HWND_TOPMOST,左边,顶边,宽度,高度, win32con.SWP_SHOWWINDOW)


@异常处理返回类型逻辑型
def 窗口_置前台(窗口句柄):
    '指定句柄设置为前台，也就是激活'
    return win32gui.SetForegroundWindow(窗口句柄)


@异常处理返回类型逻辑型
def 窗口_置后台(窗口句柄):
    '设置为后台'
    return win32gui.SetBkMode(窗口句柄, win32con.TRANSPARENT)



def _MyCallback(hwnd, extra):#回调
    windows = extra
    temp = []
    temp.append(hex(hwnd))
    temp.append(win32gui.GetClassName(hwnd))
    temp.append(win32gui.GetWindowText(hwnd))
    windows[hwnd] = temp

@异常处理返回类型逻辑型
def 窗口_取所有顶级窗口句柄():
    '返回一个字典包含 key为句柄 值为列表[16进制的啥,类名,标题]'
    windows = {}
    win32gui.EnumWindows(_MyCallback, windows)
    return windows


@异常处理返回类型逻辑型
def 取剪辑版文本():
    '获取完需要自己解码,常规用utf8带汉字的用gbk等'
    w.OpenClipboard()
    d = w.GetClipboardData(win32con.CF_TEXT)
    w.CloseClipboard()
    return d


@异常处理返回类型逻辑型
def 置剪辑版文本(内容):
    w.OpenClipboard()
    w.EmptyClipboard()
    w.SetClipboardText(内容)
    w.CloseClipboard()
    return True


@异常处理返回类型逻辑型
def 运行程序(文件路径,参数='',初始目录='',显示窗口=True):
    '启动/运行一个文件/程序'
    return win32api.ShellExecute(1, 'open',文件路径,参数,初始目录,显示窗口)


@异常处理返回类型逻辑型
def 信息框(内容,标题='提示',类型=0,图标=0):
    '''
    api创建的信息框
    :param 标题:
    :param 内容:
    :param 类型: 0.通常   1.确定 取消   2.是 否     3.是 否 取消   4.终止 重试 忽略
    :param 图标: 0.没有   1.蓝色提示   2.黄色感叹   3.红色叉叉
    :return: 出错返回0，正常返回值：1.确定 2.取消 3.中止 4.重试 5.忽略 6.是  7.否
    '''
    按钮类型 = {0:win32con.MB_OK,1:win32con.MB_OKCANCEL,2:win32con.MB_YESNO,3:win32con.MB_YESNOCANCEL,4:win32con.MB_ABORTRETRYIGNORE}
    图标类型 = {0:0,1:win32con.MB_ICONASTERISK,2:win32con.MB_ICONEXCLAMATION,3:win32con.MB_ICONERROR}
    return win32api.MessageBox(None,内容,标题,按钮类型[类型]|图标类型[图标])


@异常处理返回类型逻辑型
def 系统_建立关联文件(后缀,程序路径,图标=None):
    '后缀：.pyec,程序路径:完整的路径'
    程序名 = os.path.splitext(程序路径)[0]
    图标 = 图标 if 图标 else 程序路径
    # 创建
    key = win32api.RegOpenKey(win32con.HKEY_CLASSES_ROOT, '', 0, win32con.KEY_READ)
    win32api.RegCreateKey(key, 后缀)
    win32api.RegCreateKey(key, 程序名 + '\\BrowserFlags')
    win32api.RegCreateKey(key, 程序名 + '\\EditFlags')
    win32api.RegCreateKey(key, 程序名 + '\\DefaultIcon\\')
    win32api.RegCreateKey(key, 程序名 + '\\shell\\')
    win32api.RegCreateKey(key, 程序名 + '\\shell\\open\\command\\')
    win32api.RegCloseKey(key)
    # 写入默认值
    key = win32api.RegOpenKey(win32con.HKEY_CLASSES_ROOT, '', 0, win32con.KEY_READ)
    win32api.RegSetValue(key, '.pyec', win32con.REG_SZ, 程序名)
    win32api.RegSetValue(key, 程序名 + '\\BrowserFlags', win32con.REG_SZ, "8")
    win32api.RegSetValue(key, 程序名 + '\\EditFlags', win32con.REG_SZ, "0")
    win32api.RegSetValue(key, 程序名 + '\\DefaultIcon\\', win32con.REG_SZ,'"{}"'.format(图标))
    win32api.RegSetValue(key, 程序名 + '\\shell\\', win32con.REG_SZ, 'open')
    win32api.RegSetValue(key, 程序名 + '\\shell\\open\\command\\', win32con.REG_SZ,r'{} "%1"'.format(程序路径))
    win32api.RegCloseKey(key)
    return True


@异常处理返回类型逻辑型
def 系统_取Python目录():
    try:
        key = win32api.RegOpenKey(win32con.HKEY_LOCAL_MACHINE,'SYSTEM\\CurrentControlSet\\Control\\Session Manager\\Environment', 0,win32con.KEY_ALL_ACCESS)
        版本 = win32api.RegQueryValueEx(key, 'Path')  # 遍历注册表下目录
        win32api.RegCloseKey(key)
        匹配 = re.findall(r'\b[A-Z]:\\.*?python\b', 版本[0])
        if 匹配:
            return 匹配[0] + "\\"
    except:
        pass

    try:
        p = subprocess.Popen("where python", stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        结果 = p.stdout.readlines()[0].decode('utf8')
        if 结果.find("python") != -1:
            return os.path.split(结果.split("\n")[0])[0] + "\\"
    except:
        pass

    try:
        key = win32api.RegOpenKey(win32con.HKEY_CURRENT_USER, 'Software\\Python\\PythonCore', 0, win32con.KEY_ALL_ACCESS)
        版本 = win32api.RegEnumKeyEx(key)[0][0]  # 遍历注册表下目录
        win32api.RegCloseKey(key)
        key = win32api.RegOpenKey(win32con.HKEY_CURRENT_USER, 'Software\\Python\\PythonCore\\{}\\InstallPath'.format(版本), 0,
                                  win32con.KEY_ALL_ACCESS)
        目录 = win32api.RegQueryValueEx(key, '')[0]
        win32api.RegCloseKey(key)
        if 目录.endswith('\\'):
            return 目录
        return 目录 + "\\"
    except:
        return ''


@异常处理返回类型逻辑型
def 文件_置隐藏Ex(路径):
    '隐藏文件'
    win32api.SetFileAttributes(路径, win32con.FILE_ATTRIBUTE_HIDDEN)
    return True


@异常处理返回类型逻辑型
def 文件_置只读Ex(路径):
    '修改文件为只读'
    win32api.SetFileAttributes(路径, win32con.FILE_ATTRIBUTE_READONLY)
    return True


@异常处理返回类型逻辑型
def 文件_置系统文件Ex(路径):
    '修改文件为系统文件'
    win32api.SetFileAttributes(路径, win32con.FILE_ATTRIBUTE_SYSTEM)
    return True


@异常处理返回类型逻辑型
def 文件_置正常Ex(路径):
    '文件被隐藏等可以使用这个还原显示'
    win32api.SetFileAttributes(路径, win32con.FILE_ATTRIBUTE_NORMAL)
    return True


@异常处理返回类型逻辑型
def 宽带_拨号(宽带名称,宽带账号,宽带密码):
    dos = 'rasdial {} {} {}'.format(宽带名称,宽带账号,宽带密码)
    return False if os.system(dos) else True


@异常处理返回类型逻辑型
def 宽带_断开(宽带名称):
    dos = 'rasdial {} /disconnect'.format(宽带名称)
    return False if os.system(dos) else True



def 取屏幕宽度():
    return GetSystemMetrics(0)


def 取屏幕高度():
    return GetSystemMetrics(1)

# 取鼠标水平位置
# 取鼠标垂直位置
def 取鼠标位置():
    return pyautogui.position()


